﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SingleItemRotationMove", "A move on a multi component vector that is specified by a single index. The item at the specified index is rotated.")]
  [StorableClass]
  public class SingleItemRotationMove : MultiComponentVectorMove {
    [Storable]
    public int AffectedGroup { get; protected set; }
    [Storable]
    public int Index { get; protected set; }

    [StorableConstructor]
    protected SingleItemRotationMove(bool deserializing) : base(deserializing) { }
    protected SingleItemRotationMove(SingleItemRotationMove original, Cloner cloner)
      : base(original, cloner) {
        this.AffectedGroup = original.AffectedGroup;
        this.Index = original.Index;
    }
    public SingleItemRotationMove(int affectedBin, int index, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        AffectedGroup = affectedBin;
        Index = index;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleItemRotationMove(this, cloner);
    }

    public override IPackingSolutionEncoding GetSolutionAfterMove() {
      return GetVectorAfterMove (MultiComponentVector, AffectedGroup, Index);
    }
    public static MultiComponentVectorEncoding GetVectorAfterMove(MultiComponentVectorEncoding multiComponentVector, int affectedGroup, int index) {
      var result = multiComponentVector.Clone(new Cloner()) as MultiComponentVectorEncoding;
      result.PackingInformations[affectedGroup][index].Rotated = !result.PackingInformations[affectedGroup][index].Rotated;
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(SingleItemRotationMoveAttribute);
    }

    public override string ToString() {
      return "RM(g=" + AffectedGroup + ",i=" + Index + ")";
    }

    public override MultiComponentVectorMoveAttribute GetAttribute(double quality) {
        return new SingleItemRotationMoveAttribute(AffectedGroup, Index, 
          MultiComponentVector.PackingInformations[AffectedGroup][Index].ItemID, 
          MultiComponentVector.PackingInformations[AffectedGroup][Index].Rotated, quality);
    }

    public override bool BelongsToAttribute(MultiComponentVectorMoveAttribute attribute, bool hardCriterion) {
      SingleItemRotationMoveAttribute actualAttribute = attribute as SingleItemRotationMoveAttribute;
      if (actualAttribute != null) {
        if (hardCriterion) {
          if (AffectedGroup == actualAttribute.AffectedGroup || MultiComponentVector.PackingInformations[AffectedGroup][Index].Rotated == actualAttribute.ItemRotation)
            return true;
        } else {
          if (AffectedGroup == actualAttribute.AffectedGroup && MultiComponentVector.PackingInformations[AffectedGroup][Index].Rotated == actualAttribute.ItemRotation)
            return true;
        }
      }
      return false;
    }
  }
}